#include <iostream>
#include <fstream>
#include <cstdlib>
#include <string>

#include <pthread.h>

#include <memory>
#include <zipios++/zipios-config.h>
#include <zipios++/meta-iostreams.h>
#include <zipios++/fcollexceptions.h>
#include <zipios++/zipfile.h>

#include <FL/Fl_Window.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Wizard.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Progress.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Pixmap.H>
#include <FL/Fl_Text_Buffer.H>
#include <FL/Fl_Text_Display.H>
#include <FL/Fl_Round_Button.H>
#include <FL/Fl_Progress.H>

#include "logo.xpm"

using namespace zipios;
using std::string;
using std::cout;
using std::endl;
using std::cerr;
using std::auto_ptr ;
using std::ofstream;
using std::istream;
using std::ios_base;

class GUI
{
public:
string myname; //This is just the name of the executable
//All the widgets
Fl_Window *window;
Fl_Wizard *wiz;
Fl_Button *next;
Fl_Button *back;
Fl_Button *cancel;
Fl_Box *step_title;
Fl_Box *step_description;
Fl_Text_Display *licensebuff;
Fl_Round_Button *accept_button, *reject_button;
Fl_Input *path_input;
Fl_Button *browse_button;
Fl_Group *license_group;
Fl_Group *path_group;
Fl_Group *check_group;
Fl_Box *check_label;
Fl_Group *install_group;
Fl_Group *post_group;
Fl_Progress *gauge;

void load_gui(const char*);
void draw_buttons();
void draw_intro();
void set_wizard_widgets();
void install_files();

void license();
void path();
void check();
void install();
void post();
void intro();
void set_path(const char*);
void set_step_title(const char*, const char*);
/*callbacks*/
void set_filename(Fl_File_Chooser *browser);
void next_callback(Fl_Widget *);
void back_callback(Fl_Widget *);
void cancel_callback(Fl_Widget *);
void browse(Fl_Widget *);
void accept_cb(Fl_Widget *);
void reject_cb(Fl_Widget *);

/*overloaded static callbacks, a little drawback of fltk*/
static void accept_cb(Fl_Widget *w, void* data)
 {
 ((GUI*)data)->accept_cb(w);
 }
static void reject_cb(Fl_Widget *w, void* data)
 {
 ((GUI*)data)->reject_cb(w);
 }
static void browse(Fl_Widget *w, void* data)
 {
 ((GUI*)data)->browse(w);
 } 
static void next_callback(Fl_Widget *w, void* data)
 {
 ((GUI*)data)->next_callback(w);
 } 
static void back_callback(Fl_Widget *w, void* data)
 {
 ((GUI*)data)->back_callback(w);
 } 
static void cancel_callback(Fl_Widget *w, void* data)
 {
 ((GUI*)data)->cancel_callback(w);
 }  
static void set_filename(Fl_File_Chooser *browser, void* data)
 {
  ((GUI*)data)->set_filename(browser);
 }
};

void *extract_zip(void *);

enum //These are the steps
{
  INTRO,
  LICENSE,
  PATH,
  CHECK,
  INSTALL,
  POST
};

struct options
  {
    string AppName;
    string AppVer;
    string AppDesc;
    string CopyRight;
    string DefaultPath;
    bool with_license;
    string LicenseString;
    string InstallPath;
    int nfiles;
    string *files;
    options();
    ~options();
  };

/*some pesky global variabled*/  
struct options opt;
int current_step;
string file;
int filen;
bool DONE_EXTRACTING;

