/***************************************************************************
                          idux.cpp  -  Main idux file.
                             -------------------
    begin                : Mon Nov 3 2003
    copyright            : (C) 2003 by Miguel Morales
    email                : mrx@netlane.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "idux.h"
#include <cstdio>
#include <cstdlib>
#include <cstdarg>
#include <memory>

#include "zipios++/zipios-config.h"
#include "zipios++/meta-iostreams.h"
#include "zipios++/ziphead.h"
#include "zipios++/zipfile.h"
#include "zipios++/zipheadio.h"
#include "zipios++/zipoutputstream.h"

using namespace zipios;
struct options opt;

string temp_path = "/tmp/"; //TODO: Get temp path using automake or autoconf

void check_options();
int append_zip(const char*, const char*);
int zipfiles();
int extract_skeleton(const char*);
void toconfig();
void compile();

int main (int argc, char **argv)
{
  char ch;
  if (argc < 2)
    {
      fatal_exit ("usage: %s filename", argv[0]);
    }
  optarg = NULL;

  while ((ch = getopt (argc, argv, "")) != -1)
    {
      switch (ch)
        {
        default:
          break;
        }
    }

  if (argv[optind] == NULL)
    {
      fatal_exit ("argument parsing error!");
    }

  if (!file_exists (argv[optind]))
    {
      fatal_exit ("file does not exist");
    }

  parse_file (argv[optind]);
  check_options();
  toconfig();
  zipfiles();
  extract_skeleton(argv[0]);
  compile();
  append_zip(string(temp_path + "/wizard").c_str(), string(temp_path + "/files.zip").c_str());
  
  string tmp = "mv ";
  tmp.append(temp_path);
  tmp.append("/wizard ");
  tmp.append(opt.AppName);
  tmp.append(".wizard");
  system(tmp.c_str());
  cout << "Output: " << opt.AppName + ".wizard" << endl;
  tmp = "cd ";
  tmp.append(temp_path);
  tmp.append("; rm -f ");
  tmp.append("files.zip skeleton.cpp skeleton.h idux_config.h logo.xpm");
  cout << "Removing temporary files" << endl;
  system(tmp.c_str());
  cout << "Done." << endl;
  return 0;
}

bool
file_exists (const char *filename)
{
  bool isit;
  ifstream file;
  file.open (filename, ios::in);
  if (file.is_open ())
    isit = true;
  else
    isit = false;
  file.close ();
  return isit;
}

void
fatal_exit (const char *format, ...)
{
  va_list al;
  va_start (al, format);
  fprintf (stderr, format, va_arg (al, int));
  fprintf (stderr, "\n");
  fprintf (stderr, "Quittin.\n");
  va_end (al);
  exit (1);
}

options::~options()
{
if (opt.files)
delete files;
}

options::options ()
{
  AppName = "NULL";
  AppDesc = "";
  AppCopyright = "";
  AppVersion = "";
  string AppLicense = "";
  string LicenseString = "";
  DefaultPath = "DEFAULT";
  files = 0;
  nfiles = 0;
  ndirs = 0;
}

void
options::print_options ()
{
  FileList* temp = opt.files;
  cout << "AppName = " << AppName << endl;
  cout << "AppCopyright = " << AppCopyright << endl;
  cout << "AppVersion = " << AppVersion << endl;
  cout << "DefaultPath = " << DefaultPath << endl;
  cout << "AppLicense = " << AppLicense << endl;
  cout << "nfiles = " << nfiles << endl;
  cout << "ndirs = " << ndirs << endl;

  while(temp)
    {
      cout << files->name << endl;
      cout << files->path << endl;
      cout << files->flags << endl;
      cout << endl;
      temp = temp->next;
    }
    for (int x = 0; x < ndirs; x++)
    {
      cout << "directories[" << x << "] = " << directories[x] << endl;
    }
  cout << endl;
}

void toconfig()
{
  string path;
  ofstream file;
  FileList *temp = opt.files;
  path = temp_path;
  path.append("/idux_config.h");
  cout << "Writing config file..." << endl;

  file.open (path.c_str(), ios::out | ios::trunc);
  if (!file.is_open ())
    fatal_exit("Couldn't write to: /tmp/idux_config.h");

  file << "#define APPNAME \"" << opt.AppName << "\"" << endl;
  file << "#define APPVER \"" << opt.AppVersion << "\"" << endl;
  file << "#define APPDESCRIPTION \"" << opt.AppDesc << "\"" << endl;
  file << "#define COPYRIGHT \"" << opt.AppCopyright << "\"" << endl;
  file << "#define DEFAULTPATH \"" << opt.DefaultPath << "\"" << endl;
  file << "#define NFILES " << opt.nfiles << endl;

  file << "string idux_files[NFILES * 3] = {";
  while(temp)
  {
  file << "\"" << (opt.files)->name << "\", \"" << (opt.files)->path << "\", \"" << (opt.files)->flags << "\", ";
  temp = temp->next;
  }
 
  file << "};" << endl;

  if (opt.ndirs >= 1)
    {
      file << "#define DIRS \"";
      for (int x = 0; x < opt.ndirs; x++)
        file << opt.directories[x] << " ";
      file << "\"" << endl;
    }
  if ((opt.LicenseString).length() > 0)
    {
      file << "#define WITHLICENSE 1" << endl;
      file << "#define LICENSE_STRING \"" <<  opt.LicenseString << "\"" << endl;
    }

  file.close();
}

void check_options()
{
  FileList* temp = opt.files;
  if ((opt.AppName).length() < 1)
    fatal_exit("AppName not defined!");
while(temp)
	{
        if(!file_exists((temp->name).c_str()))
        	fatal_exit("%s does not exist!", (temp->name).c_str());
        temp = temp->next;
        }
}

int
append_zip (const char *outputexec, const char *inputzip)
{
  /*Taken from the zipios++ appendzip source*/
  ofstream
  exef (outputexec, ios::app | ios::binary);
  if (!exef)
    {
    fatal_exit("Could not open %s!", outputexec);
    return 1;
    }
  ifstream
  zipf (inputzip, ios::in | ios::binary);
  if (!zipf)
  {
  fatal_exit("Could not open %s!", inputzip);
  return 1;
  }
  uint32
  zip_start = exef.tellp ();
  exef << zipf.rdbuf ();
  writeUint32 (zip_start, exef);
  exef.close ();
  zipf.close ();
  cout << "Appended: " << inputzip << " To: " << outputexec << endl;
  return 0;
}

int zipfiles()
{
  cout << "Zipping files..." << endl;
  FileList *temp = opt.files;
  try {
    ZipOutputStream zip(temp_path + "/files.zip" );
  
  while(temp)
  	{
  	cout << "Compressing: " << temp->name << endl;
        zip.putNextEntry( ZipCDirEntry(temp->name));
	ifstream ifs((temp->name).c_str(), ios::in | ios::binary );
  	zip << ifs.rdbuf() ; 
  	ifs.close();
        temp = temp->next;
        }
  zip.close();
  }
  catch( exception &excp ) {
    cerr << "Exception caught while zipping files." << endl ;
    cerr << excp.what() << endl ;
    return 1;
  }
return 0;
}

void compile()
{
/*I am still unsure as how to do this, for now this will do*/
string com = "cd ";
com.append(temp_path);
com.append("; g++ ");
com.append("-s -static -DHAVE_STD_IOSTREAM -DUSE_STD_IOSTREAM skeleton.cpp -o wizard -L/usr/X11R6/lib -lfltk -lX11 -lzipios -lz -lpthread");
cout << "Compiling..." << endl;
if (!system(com.c_str()))
	cout << "Compilation might have failed." << endl;
system(com.c_str());
}

int extract_skeleton(const char* myname)
{
  ZipFile zip;
  ConstEntries entries;
  ConstEntryPointer current;
  string current_file;
  cout << "Extracting skeleton..." << endl;
  try
    {
      zip = ZipFile::openEmbeddedZipFile(myname);
    }
  catch(...)
    {
      fatal_exit("Couldn't open embedded zip in: %s\n", myname);
      return 1;
    }
  if (!zip.isValid())
    {
      fatal_exit("Invalid zip file!");
      return 1;
    }

entries = zip.entries();
ConstEntries::iterator it ;
ofstream ofs;
string temp;
for( it = entries.begin() ; it != entries.end() ; it++)
  {
      current_file = (*(*it)).getName();
      current = zip.getEntry(current_file, FileCollection::IGNORE ) ;
      auto_ptr< istream > is( zip.getInputStream(current) ) ;
      temp = temp_path;
      temp.append("/");
      temp.append(current_file);
      ofs.open(temp.c_str(), ios::binary);
      ofs << is->rdbuf();
      ofs.close();
     
  }

zip.close();
}
