/***************************************************************************
                          parser.cpp  -  Takes care of parsing the file.
                             -------------------
    begin                : Mon Nov 3 2003
    copyright            : (C) 2003 by Miguel Morales
    email                : mrx@netlane.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "idux.h"
string make_file_to_string (string);
int get_section (string temp);
string getsub (string, const char *, const char *);
bool check_value (string, int);
bool set_SETUP_value (string, string);
bool set_FILES_value (string, string, string, string);
void add_file(FileList *);

void parse_file (const char *filename)
{
  int linenum = 0;
  int parameter = 0;
  int current_section = 0;
  string line;
  ifstream file;
  file.open (filename, ios::in);
  while (getline (file, line))
    {
      linenum++;
      if (line.length () < 1)
        continue;
      if (line[0] == '#')
        continue;

      if (line.find ('[') != string::npos && line.find (']') != string::npos)
        {
          if ((current_section = get_section (line)) == ERROR)
            fatal_exit
            ("Parse Error at line: %d.  Section name not recognized.",
             linenum);
          continue;
        }
      if (!check_value (line, current_section))
        fatal_exit ("Parse error at line: %d.", linenum);
    }
  file.close ();

  //Load license file
  if ((opt.AppLicense).length () > 0)
    {
      if (!file_exists ((opt.AppLicense).c_str ()))
        fatal_exit ("File: %s does not exist!", (opt.AppLicense).c_str ());
      opt.LicenseString = make_file_to_string (opt.AppLicense);
    }

}

//This function checks the value for the given section
bool check_value (string temp, int section)
{
  string param_name = "";
  string value = "";
  string destdir = "";
  string flag = "";

  switch (section)
    {
    case SETUP:
      param_name = getsub (temp, "", "=");
      if (param_name == "-2")
        return false;
      value = temp.substr (temp.find ('=') + 1, temp.length ());
      return set_SETUP_value (param_name, value);
      // if (value.length() < 1) return false;
      break;


    case FILES:
      param_name = getsub (temp, "", ":");
      value = getsub (temp, ": ", ";");
      if (value == "-1")
        return false;
      destdir = getsub (temp, "DestDir: ", ";");
      if (destdir == "-2")
        return false;

      if (temp.find ("Flag: ") != string::npos)
        flag = getsub(temp, "Flag: ", ";");
      if (param_name == "-2")
        return false;
      if (value.length () < 1)
        return false;
      return set_FILES_value (param_name, value, destdir, flag);
      break;


    default:
      return false;
    }
  return true;
}

bool set_FILES_value (string param, string value, string destdir, string flag)
{
  
  if (param == "File")
    {
     FileList *temp = new FileList; 
      opt.nfiles++;
      temp->name = value;
      if (destdir == "-1")
        temp->path = opt.DefaultPath;
      else
        temp->path = destdir;

      if (flag.length () < 1)
        temp->flags = "none";
      else if (flag == "binary")
        temp->flags = flag;
      else
        return false;
      add_file(temp);  
      return true;
    }

  if (param == "SubDir")
    {
      if (value == "-2")
        return false;
      opt.ndirs++;
      opt.directories[opt.dircounter++] = value;
      return true;
    }
  return false;
}

bool
set_SETUP_value (string param, string value)
{
  if (param == "AppName")
    {
      opt.AppName = value;
      return true;
    }
  if (param == "AppDesc")
    {
      opt.AppDesc = value;
      return true;
    }
  if (param == "AppCopyright")
    {
      opt.AppCopyright = value;
      return true;
    }
  if (param == "AppVer")
    {
      opt.AppVersion = value;
      return true;
    }
  if (param == "DefaultPath")
    {
      opt.DefaultPath = value;
      return true;
    }
  if (param == "AppLicense")
    {
      opt.AppLicense = value;
      return true;
    }
  return false;
}

int
get_section (string temp)
{
  temp = getsub (temp, "[", "]");
  if (temp == "SETUP")
    return SETUP;
  if (temp == "FILES")
    return FILES;

  return ERROR;
}

string
getsub (string full, const char sub1[], const char sub2[])
{
  int length = full.length ();
  unsigned int loc = full.find (sub1, 0);
  if (loc != string::npos)
    {
      int indexlength = strlen (sub1);
      loc = loc + indexlength;

      string str2 = full.substr (loc, length);

      unsigned int loc2 = str2.find (sub2, 0);
      if (loc2 != string::npos)
        {
          string substring = str2.substr (0, loc2);
          return substring;
        }
      else
        {
          return "-2";		// Couldn't find second parameter.
        }
    }
  else
    {
      return "-1";		//Couldn't find first parameter.
    }
}

string
make_file_to_string (string filename)
{
  /*Makes the contents of a file into a C-style string.*/
  ifstream file;
  string line;
  string full = "";
  if (!file_exists (filename.c_str ()))
    fatal_exit ("File: %s, does not exist!", filename.c_str ());

  file.open (filename.c_str (), ios::in);
  while (getline (file, line))
    {
      full.append (line);
      full.append ("\n");
    }

  for (size_t x = 0; x < full.length (); x++)
    {
      if (full[x] == '"')
        {
          full.insert (x, "\\");
          x++;
        }

      if (full[x] == '\n')
        {
          full.insert(x, "\\n\\");
          x = x+3;
        }
    }

  file.close ();
  return full;
}

void add_file(FileList *data)
{
data->next = 0;
if (opt.files == 0)
	{
        opt.files = data;
        return;
        }
FileList *current = opt.files;
while(current->next)
	current = current->next;
current->next = data;
}
